package com.pavelpetkevich.bcwork.fragment;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.pavelpetkevich.bcwork.R;
import com.pavelpetkevich.bcwork.adapters.RecyclerViewPlaceAdapter;
import com.pavelpetkevich.bcwork.data.ItemPlaceRecyclerView;
import com.pavelpetkevich.bcwork.data.PlaceList;
import com.pavelpetkevich.bcwork.dialogs.ShowDialog;
import com.pavelpetkevich.bcwork.service.ApiService;
import com.pavelpetkevich.bcwork.service.ControlServices;
import com.pavelpetkevich.bcwork.service.RetroClient;

import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class PlacesFragment extends Fragment implements SwipeRefreshLayout.OnRefreshListener{
    private static final String PLACES_URL = "https://api.myjson.com";

    private ArrayList<ItemPlaceRecyclerView> itemList;
    private RecyclerViewPlaceAdapter adapter;

    @BindView(R.id.my_recycler_view)
    RecyclerView recyclerView;
    @BindView(R.id.imgSomeWrong)
    ImageView imgSomeWrong;
    @BindView(R.id.tv_error_title_primary)
    TextView tv_error_title_primary;
    @BindView(R.id.tv_error_message_secondary)
    TextView tv_error_message_secondary;

    @BindView(R.id.swipe_refresh_layout)
    SwipeRefreshLayout swipeRefreshLayout;

    Activity mActivity;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        mActivity = context instanceof Activity ? (Activity) context : null;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        itemList = new ArrayList<>();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.recycler_view, container, false);
        ButterKnife.bind(this, view);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        new TakeDates().execute();
        setSwipeRefreshLayout(true);
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!ControlServices.controlNetworkStateService(mActivity)) {
            showErrorMessage(getString(R.string.place_frag_error_msg_network_state_title), getString(R.string.place_frag_error_msg_network_state_message), R.drawable.ic_error_no_wifi);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_places_text), mActivity);
    }

    @Override
    public void onPause() {
        super.onPause();
        setSwipeRefreshLayout(false);
    }

    @Override
    public void onRefresh() {
        adapter.clearRecyclerView();
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                populateFromJSON();
                setSwipeRefreshLayout(false);
            }
        }, 2000);
    }

    /**
     * Take dates by AsyncTask.
     * Cause for using is thread sleep and show progress dialog on postExecute call populateJSON.
     */
    @SuppressLint("StaticFieldLeak")
    class TakeDates extends AsyncTask<Void, Void, Void> {

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            ShowDialog.showProgressDialog(true, getString(R.string.progress_dialog_places_text), mActivity);
        }

        @Override
        protected Void doInBackground(Void... voids) {
            try {
                Thread.sleep(2200);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);
            if(isAdded())
                populateFromJSON();
        }

    }

    /**
     * Populate from JSON Api Service.
     * Take some places with information about them from API.
     */
    private void populateFromJSON() {
        if (ControlServices.controlNetworkStateService(mActivity)) {
            ApiService api = RetroClient.getApiService(PLACES_URL);

            Call<PlaceList> call = api.getMyJSON();
            call.enqueue(new Callback<PlaceList>() {
                @Override
                public void onResponse(@NonNull Call<PlaceList> call, @NonNull Response<PlaceList> response) {
                    if (response.isSuccessful()) {
                        itemList = response.body().getItemList();
                        adapter = new RecyclerViewPlaceAdapter(itemList, mActivity);
                        recyclerView.setAdapter(adapter);
                        recyclerView.setHasFixedSize(true);
                        recyclerView.setLayoutManager(new LinearLayoutManager(mActivity));
                        showPlaces();
                    } else {
                        showErrorMessage(getString(R.string.place_frag_error_msg_response_title), getString(R.string.place_frag_error_msg_response_message), R.drawable.ic_error_uknown);
                    }
                }

                @Override
                public void onFailure(@NonNull Call<PlaceList> call, @NonNull Throwable t) {
                    showErrorMessage(getString(R.string.place_frag_error_msg_failure_parse_title), getString(R.string.place_frag_error_msg_failure_parse_message_1) + t.getMessage() + getString(R.string.place_frag_error_msg_failure_parse_message_2), R.drawable.ic_error_uknown);
                }
            });
        } else {
            showErrorMessage(getString(R.string.place_frag_error_msg_network_state_title), getString(R.string.place_frag_error_msg_network_state_message), R.drawable.ic_error_no_wifi);
        }
    }

    /**
     * Show error message whenever is failed
     *
     * @param title - title of error message dialog
     * @param msg   - main message with some problem/error
     * @param icon  - image of know and unknown problems/errors
     */
    private void showErrorMessage(String title, String msg, int icon) {
        recyclerView.setVisibility(View.INVISIBLE);

        tv_error_title_primary.setVisibility(View.VISIBLE);
        tv_error_message_secondary.setVisibility(View.VISIBLE);
        imgSomeWrong.setVisibility(View.VISIBLE);

        imgSomeWrong.setImageResource(icon);
        tv_error_title_primary.setText(title);
        tv_error_message_secondary.setText(msg);

        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_places_text), mActivity);
    }

    /**
     * Set up swipe refresh layout.
     *
     * @param set - this boolean param is for control if the swipeRefresh may be available or not.
     */
    private void setSwipeRefreshLayout(boolean set) {
        if (set) {
            swipeRefreshLayout.setOnRefreshListener(this);
            swipeRefreshLayout.setColorSchemeResources(R.color.swipeRefreshAccent, R.color.swipeRefreshAccent2, R.color.swipeRefreshAccent3, R.color.swipeRefreshAccent4);
        } else {
            swipeRefreshLayout.setRefreshing(false);
            swipeRefreshLayout.destroyDrawingCache();
            swipeRefreshLayout.clearAnimation();
        }
    }

    /**
     * Show place if response is successful
     */
    private void showPlaces() {
        tv_error_title_primary.setVisibility(View.INVISIBLE);
        tv_error_message_secondary.setVisibility(View.INVISIBLE);
        imgSomeWrong.setVisibility(View.INVISIBLE);

        recyclerView.setVisibility(View.VISIBLE);
        ShowDialog.showProgressDialog(false, getString(R.string.progress_dialog_places_text), mActivity);
    }
}
